part of fraction;

class Fraction {
  late BigInt numerator;
  late BigInt denominator;

  Fraction(this.numerator, {BigInt? denominator}) {
    this.denominator = denominator ?? BigInt.one;
  }

  // performs floor division
  BigInt quotient() {
    assert(denominator != BigInt.zero, 'denominator can not BIGINT 0');
    if (denominator == BigInt.one) {
      return numerator;
    }
    return BigInt.from(numerator / denominator);
  }

  Fraction tryParseFraction(BigInt fractionish) {
    return Fraction(fractionish);
  }

  // remainder after floor division
  Fraction remainder() {
    final number = numerator % denominator;
    return Fraction(number, denominator: denominator);
  }

  Fraction invert() {
    return Fraction(denominator, denominator: numerator);
  }

  Fraction add(Fraction otherParsed) {
    if (denominator == otherParsed.denominator) {
      return Fraction(numerator + otherParsed.numerator,
          denominator: denominator);
    }
    return Fraction(
        (numerator * otherParsed.denominator) +
            (otherParsed.numerator * denominator),
        denominator: denominator * otherParsed.denominator);
  }

  Fraction subtract(Fraction otherParsed) {
    if (denominator == otherParsed.denominator) {
      return Fraction(numerator - otherParsed.numerator,
          denominator: denominator);
    }
    return Fraction(
        (numerator * otherParsed.denominator) -
            (otherParsed.numerator * denominator),
        denominator: denominator * otherParsed.denominator);
  }

  bool lessThan(Fraction otherParsed) {
    return (numerator * otherParsed.denominator) <
        (otherParsed.numerator * denominator);
  }

  bool equalTo(Fraction otherParsed) {
    return (numerator * otherParsed.denominator) ==
        (otherParsed.numerator * denominator);
  }

  bool greaterThan(Fraction otherParsed) {
    return (numerator * otherParsed.denominator) >
        (otherParsed.numerator * denominator);
  }

  Fraction multiply(Fraction otherParsed) {
    return Fraction((numerator * otherParsed.numerator),
        denominator: (denominator * otherParsed.denominator));
  }

  Fraction divide(Fraction otherParsed) {
    return Fraction((numerator * otherParsed.denominator),
        denominator: (denominator * otherParsed.numerator));
  }

  String toSignificantNative(
    int significantDigits, {
    NumberFormat? format,
    RoundingMode? rounding,
  }) {
    invariant(significantDigits > 0, '$significantDigits is not positive.');

    final rm = rounding ?? RoundingMode.roundHalfUp;

    final numeratorConvertBig = Big(numerator);//convert BigInt to Big
    final denominatorConvertBig = Big(denominator);//convert BigInt to Big

    return Decimal.parse(numeratorConvertBig.div(denominatorConvertBig).toStringAsPrecision(significantDigits,rm)).toString();
  }

  String toFixedNative(
    int decimalPlaces, {
    NumberFormat? format,
    RoundingMode? rounding,
  }) {
    invariant(decimalPlaces >= 0, '$decimalPlaces is negative.');

    final rm = rounding ?? RoundingMode.roundHalfUp;

    final numeratorConvertBig = Big(numerator);//convert BigInt to Big
    final denominatorConvertBig = Big(denominator);//convert BigInt to Big

    return numeratorConvertBig.div(denominatorConvertBig).toStringAsFixed(dp: decimalPlaces,rm: rm);
  }

  @override
  bool operator ==(Object other) {
    if (other is Fraction) {
      return numerator == other.numerator && denominator == other.denominator;
    } else {
      return false;
    }
  }

  @override
  int get hashCode => numerator.hashCode ^ denominator.hashCode;

  /// Helper method for converting any super class back to a fraction
  Fraction asFraction() {
    return Fraction(numerator, denominator: denominator);
  }
}
