import 'package:pancake_sdk/core/entities/index.dart';
import 'package:pancake_sdk/core/entities/token/tokenAmount.dart';
import 'package:pancake_sdk/core/utils/index.dart';
import 'package:pancake_sdk/v2/constants.dart';
import 'package:pancake_sdk/v2/pair.dart';
import 'package:web3dart/web3dart.dart';

final TOKEN_DECIMALS_CACHE = {
  ChainId.MAINNET: {
    '0xE0B7927c4aF23765Cb51314A0E0521A9645F0E2A': 9 // DGD
  }
};

class Fetcher {
  static Future<Token> fetchTokenData(
      Web3Client web3client, int chainId, EthereumAddress address,
      {String? symbol, required String name}) async {

    var contract = DeployedContract(PancakeSwapV2Pair, address);

    var resp = await web3client.call(
        contract: contract,
        function: contract.function('decimals'),
        params: []);

    String symbolToken = symbol ?? '';

    if(symbol == null){
      var resS = await web3client.call(
          contract: contract,
          function: contract.function('symbol'),
          params: []);

      symbolToken = resS.first.toString();
    }

    return Token(chainId, address, resp.first.toInt(),
        symbol: symbolToken, name: name);
  }

  static Future<Pair> fetchPairData(
      Web3Client web3client, Token tokenA, Token tokenB) async {
    invariant(tokenA.chainId == tokenB.chainId, 'CHAIN_ID');

    var balances = tokenA.sortsBefore(tokenB) ? [tokenA, tokenB] : [tokenB, tokenA];
    var pairAddress = Pair.getAddress(balances[0], balances[1]);

    var contract = DeployedContract(PancakeSwapV2Pair, pairAddress!);

    var resp = await web3client.call(
        contract: contract,
        function: contract.function('getReserves'),
        params: []);
    var tokenAmount0 = TokenAmount(balances[0], resp[0]);
    var tokenAmount1 = TokenAmount(balances[1], resp[1]);

    return Pair(tokenAmount0, tokenAmount1);
  }
}
