import 'dart:typed_data';
import 'package:pancake_sdk/core/constants.dart';
import 'package:pancake_sdk/core/utils/index.dart';
import 'package:pancake_sdk/v2/constants.dart';
import 'package:pancake_sdk/v2/trade.dart';
import 'package:web3dart/credentials.dart';
import 'package:web3dart/crypto.dart';

void validateSolidityTypeInstance(BigInt value, SolidityType solidityType) {
  invariant(value >= BigInt.zero, '$value is not a solidityType.');
  invariant(value <= MaxUint256, '$value is not a solidityType.');
}

EthereumAddress getCreate2Address(
    String from, String salt, String initCodeHash) {
  if (hexToBytes(initCodeHash).length != 32) {
    throw ArgumentError("initCodeHash must be 32 bytes $initCodeHash");
  }

  var startBytes = hexToBytes('0xff');
  var factoryBytes = hexToBytes(from);
  var saltBytes = keccak256(hexToBytes(salt));
  var initCodeHashBytes = hexToBytes(initCodeHash);

  return EthereumAddress(keccak256(Uint8List.fromList(
          startBytes + factoryBytes + saltBytes + initCodeHashBytes))
      .sublist(12));
}

// given an array of items sorted by `comparator`, insert an item into its sort index and constrain the size to
// `maxSize` by removing the last item
sortedInsertTrade(List<Trade> items, Trade add, int maxSize) {
  invariant(maxSize > 0, 'MAX_SIZE_ZERO');
// this is an invariant because the interface cannot return multiple removed items if items.length exceeds maxSize
//   invariant(items.length <= maxSize, 'ITEMS_SIZE');

// short circuit first item add
  if (items.isEmpty) {
    items.add(add);
    return null;
  } else {
    final isFull = items.length == maxSize;
// short circuit if full and the additional item does not come before the last item
    if (isFull && (tradeComparator(items[items.length - 1], add) <= 0)) {
      return add;
    }

    int lo = 0;
    int hi = items.length;

    while (lo < hi) {
      final mid = (lo + hi) >>> 1;
      if (tradeComparator(items[mid], add) <= 0) {
        lo = mid + 1;
      } else {
        hi = mid;
      }
    }
    items = splice(items, lo, 0, add);
    if (isFull) {
      return items.last;
    }
    return null;
  }
}

List<T> splice<T>(List<T> list, int index,
    [num howMany = 0, /*<T | List<T>>*/ elements]) {
  var endIndex = index + howMany.truncate();
  list.removeRange(index, endIndex >= list.length ? list.length : endIndex);
  if (elements != null) {
    list.insertAll(index, elements is List<T> ? elements : <T>[elements]);
  }
  return list;
}