import 'package:pancake_sdk/core/constants.dart';
import 'package:pancake_sdk/core/entities/fraction/index.dart';
import 'package:pancake_sdk/core/entities/index.dart';
import 'package:pancake_sdk/core/entities/token/tokenAmount.dart';
import 'package:pancake_sdk/v2/constants.dart';
import 'package:test/test.dart';
import 'package:web3dart/web3dart.dart';

void main() {
  group('CurrencyAmount', () {
    final addressOne = EthereumAddress.fromHex('0x0000000000000000000000000000000000000001');

    group('#currentCurrency', () {
      test('test equal current', () {
        final amountA = CurrencyAmount.ether(BigInt.from(100),);
        final amountB = CurrencyAmount.ether(BigInt.from(100),);
        expect(amountA.currency, amountB.currency);
      });
    });

    group('constructor', () {
      test('works', () {
        final token = Token(ChainId.MAINNET.value, addressOne, 18);
        final amount = TokenAmount(token, BigInt.from(100));
        expect(amount.raw, equals(BigInt.from(100)));
      });
    });

    group('#currencyEquals', () {
      test('test TokenAmount equals', () {
        final tokenAmountA = TokenAmount(Token(ChainId.MAINNET.value, addressOne, 18), MaxUint256);
        final tokenAmountB = TokenAmount(Token(ChainId.MAINNET.value, addressOne, 18), MaxUint256);
        expect(tokenAmountA, tokenAmountB);
      });
    });

    group('.ether', () {
      test('produces ether amount', () {
        final amount = CurrencyAmount.ether(BigInt.from(100),);
        expect(amount.raw, equals(BigInt.from(100)));
      });
    });

    test('token amount can be max uint256', () {
      final amount = TokenAmount(Token(ChainId.MAINNET.value, addressOne, 18), MaxUint256);
      expect(amount.raw, MaxUint256);
    });
  });
}
