import 'package:pancake_sdk/core/entities/fraction/index.dart';
import 'package:test/test.dart';

void main() {

  group('#Fraction', () {

    group('#currencyEquals', () {
      test('test Fraction equals', () {
        final fractionA = Fraction(BigInt.from(8), denominator: BigInt.from(3));
        final fractionB = Fraction(BigInt.from(8), denominator: BigInt.from(3));
        expect(fractionA, fractionB);
      });
    });

    test('quotient', () {
      expect(
          Fraction(BigInt.from(8), denominator: BigInt.from(3)).quotient().sign,
          BigInt.from(2).sign); // one below
    });

    test('remainder', () {
      final numeratorData1 = BigInt.from(16) % BigInt.from(5);
      expect(numeratorData1, BigInt.from(1));
    });

    test('invert', () {
              expect(
                Fraction(BigInt.from(5), denominator: BigInt.from(10)).invert().numerator,
                BigInt.from(10));
    });

    test('add', ()  {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10))
                    .add(Fraction(BigInt.from(4), denominator: BigInt.from(12)))
                    .denominator,
                Fraction(BigInt.from(52), denominator: BigInt.from(120))
                    .denominator,
              );
    });

    test('subtract', (){
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).subtract(Fraction(BigInt.from(4), denominator: BigInt.from(12))).numerator,
                Fraction(BigInt.from(-28), denominator: BigInt.from(120)).numerator,
              );
    });

    test('lessThan', () {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).lessThan(Fraction(BigInt.from(4), denominator: BigInt.from(12))),
                true,
              );
    });

    test('equalTo', () {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).equalTo(Fraction(BigInt.from(4), denominator: BigInt.from(12))),
                false,
              );
    });

    test('greaterThan', () {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).greaterThan(Fraction(BigInt.from(4), denominator: BigInt.from(12))),
                false,
              );
    });

    test('multiplty', () {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).multiply(Fraction(BigInt.from(4), denominator: BigInt.from(12))).numerator,
                Fraction(BigInt.from(4), denominator: BigInt.from(120)).numerator,
              );
    });

    test('divide', () {
              expect(
                Fraction(BigInt.from(1), denominator: BigInt.from(10)).divide(Fraction(BigInt.from(4), denominator: BigInt.from(12))).numerator,
                Fraction(BigInt.from(12), denominator: BigInt.from(40)).numerator,
              );
    });

    test('asFraction', () {
      final f = Fraction(BigInt.from(1), denominator: BigInt.from(2));
      expect(f.asFraction().numerator, f.numerator);
    });

    test('toFixed', () {
              expect(
                  Fraction(BigInt.from(154), denominator: BigInt.from(10000)).toFixedNative(4),
                  '0.0154');
    });
  });
}
