import 'package:flutter_test/flutter_test.dart';
import 'package:pancake_sdk/core/entities/fraction/index.dart';

void main() {
  group('Percent', () {
    group('constructor', () {
      test('defaults to 1 denominator', () {
        expect(
            Percent(BigInt.one),
            Percent(BigInt.one, denominator: BigInt.one));
      });
    });

    group('#add', () {
      test('returns a percent', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(100)) + Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(3), denominator: BigInt.from(100)));
      });

      test('different denominators', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(25)) + Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(150), denominator: BigInt.from(2500)));
      });
    });

    group('#subtract', () {
      test('returns a percent', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(100)) - Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(-1), denominator: BigInt.from(100)));
      });

      test('different denominators', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(25)) - Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(50), denominator: BigInt.from(2500)));
      });
    });

    group('#multiply', () {
      test('returns a percent', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(100)) * Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(2), denominator: BigInt.from(10000)));
      });

      test('different denominators', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(25)) * Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(2), denominator: BigInt.from(2500)));
      });
    });

    group('#divide', () {
      test('returns a percent', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(100)) /  Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(100), denominator: BigInt.from(200)));
      });

      test('different denominators', () {
        expect(
            Percent(BigInt.one, denominator: BigInt.from(25))/ Percent(BigInt.two, denominator: BigInt.from(100)),
            Percent(BigInt.from(100), denominator: BigInt.from(50)));
      });
    });

    group('#toSignificant', () {
      test('returns the value scaled by 100', () {
        expect(
            Percent(BigInt.from(154), denominator: BigInt.from(10000)).toSignificant(3),
            '1.54');
      });
    });

    group('#toFixed', () {
      test('returns the value scaled by 100', () {
        expect(
            Percent(BigInt.from(154), denominator: BigInt.from(10000)).toFixed(2),
            '1.54');
      });
    });
  });
}
