import 'package:pancake_sdk/core/entities/fraction/index.dart';
import 'package:pancake_sdk/core/entities/index.dart';
import 'package:test/test.dart';
import 'package:web3dart/credentials.dart';

void main() {
  group('#CurrencyAmount', () {
    final addressZero =
        EthereumAddress.fromHex('0x0000000000000000000000000000000000000000');
    final addressOne =
        EthereumAddress.fromHex('0x0000000000000000000000000000000000000001');

    final t0 = Token(1, addressZero, 18);
    final t0_6 = Token(1, addressZero, 6);
    final t1 = Token(1, addressOne, 18);

    group('#constructor', () {
      test('array format works', () {
        final price =
            Price.fromContractor(t0, t1, BigInt.from(1), BigInt.from(54321));

        expect(price.toSignificant(significantDigits: 5), '54321');
        expect(price.baseCurrency, t0);
        expect(price.quoteCurrency, t1);
      });
    });

    group('#currencyEquals', () {
      test('test price equals', () {
        final priceA =
        Price.fromContractor(t0, t1, BigInt.from(1), BigInt.from(54321));
        final priceB =
        Price.fromContractor(t0, t1, BigInt.from(1), BigInt.from(54321));

        expect(priceA, priceB);
      });
    });

    group('#quote', () {
      test('returns correct value', () {
        final price =
            Price.fromContractor(t0, t1, BigInt.from(1), BigInt.from(5));
        expect(
            price
                .quote(CurrencyAmount.fromRawAmount(t0, BigInt.from(10)))
                .denominator,
            CurrencyAmount.fromRawAmount(t1, BigInt.from(50)).denominator);
        expect(
            price
                .quote(CurrencyAmount.fromRawAmount(t0, BigInt.from(10)))
                .numerator,
            CurrencyAmount.fromRawAmount(t1, BigInt.from(50)).numerator);
      });
    });

    group('#toSignificant', () {
      test('no decimals', () {
        final price =
            Price.fromContractor(t0, t1, BigInt.from(123), BigInt.from(456));
        expect(price.toSignificant(significantDigits: 4), '3.707');
      });

      test('no decimals flip ratio', () {
        final price =
            Price.fromContractor(t0, t1, BigInt.from(456), BigInt.from(123));
        expect(price.toSignificant(significantDigits: 4), '0.2697');
      });

      test('no decimals flip ratio', () {
        final price =
            Price.fromContractor(t0_6, t1, BigInt.from(123), BigInt.from(456));
        expect(
            price.toSignificant(significantDigits: 4), '0.000000000003707');
      });

      test('with decimal difference flipped', () {
        final price =
            Price.fromContractor(t0_6, t1, BigInt.from(456), BigInt.from(123));
        expect(
            price.toSignificant(significantDigits: 4), '0.0000000000002697');
      });

      test('with decimal difference flipped base quote flipped', () {
        final price =
            Price.fromContractor(t1, t0_6, BigInt.from(456), BigInt.from(123));
        expect(price.toSignificant(significantDigits: 4), '269700000000');
      });
    });


  });
}
