import 'package:flutter_test/flutter_test.dart';
import 'package:pancake_sdk/core/entities/index.dart';
import 'package:pancake_sdk/core/entities/token/tokenAmount.dart';
import 'package:pancake_sdk/v2/constants.dart';
import 'package:pancake_sdk/v2/pair.dart';
import 'package:pancake_sdk/v2/route.dart';
import 'package:web3dart/web3dart.dart';

void main() {
  final token0 = Token(ChainId.MAINNET.value,
      EthereumAddress.fromHex('0x0000000000000000000000000000000000000001'), 18,
      symbol: 't0');
  final token1 = Token(ChainId.MAINNET.value,
      EthereumAddress.fromHex('0x0000000000000000000000000000000000000002'), 18,
      symbol: 't1');
  final weth = Token.WETH[ChainId.MAINNET.value]!;
  final pair_0_1 = Pair(TokenAmount(token0, BigInt.from(100)),
      TokenAmount(token1, BigInt.from(200)));
  final pair0Weth = Pair(TokenAmount(token0, BigInt.from(100)),
      TokenAmount(weth, BigInt.from(100)));
  final pair1Weth = Pair(TokenAmount(token1, BigInt.from(175)),
      TokenAmount(weth, BigInt.from(100)));

  test('constructs a path from the tokens', () async {
    final route = Route([pair_0_1], token0);
    expect(route.pairs, [pair_0_1]);
    expect(route.path, [token0, token1]);
    expect(route.input, token0);
    expect(route.output, token1);
    expect(route.chainId, ChainId.MAINNET);
  });

  test('can have a token as both input and output', () async {
    final route = Route([pair0Weth, pair_0_1, pair1Weth], weth);
    expect(route.pairs, [pair0Weth, pair_0_1, pair1Weth]);
    expect(route.input, weth);
    expect(route.output, weth);
  });

  test('supports ether input', () async {
    final route = Route([pair0Weth], ETHER);
    expect(route.pairs,[pair0Weth]);
    expect(route.input,ETHER);
    expect(route.output,token0);
  });

  test('supports ether output', () async {
    final route = Route([pair0Weth], token0,output: ETHER);
    expect(route.pairs, [pair0Weth]);
    expect(route.input, token0);
    expect(route.output, ETHER);
  });
}
