import 'package:pancake_sdk/core/entities/index.dart';
import 'package:pancake_sdk/v2/constants.dart';
import 'package:test/test.dart';
import 'package:web3dart/credentials.dart';

void main() {
  group('Token', () {
    final addressOne =
        EthereumAddress.fromHex('0x0000000000000000000000000000000000000001');
    final addressTwo =
        EthereumAddress.fromHex('0x0000000000000000000000000000000000000002');

    group('equals', () {
      test('false if address differs', () {
        expect(
            Token(ChainId.MAINNET.value, addressOne, 18) ==
                Token(ChainId.MAINNET.value, addressTwo, 1),
            equals(false));
      });
      test('false if chain differs', () {
        expect(
            Token(ChainId.TESTNET.value, addressOne, 18) ==
                Token(ChainId.MAINNET.value, addressTwo, 1),
            equals(false));
      });

      test('true if only decimals differs', () {
        expect(
            Token(ChainId.MAINNET.value, addressOne, 9) ==
                Token(ChainId.MAINNET.value, addressOne, 1),
            equals(true));
      });
      test('true if address is the same', () {
        expect(
            Token(ChainId.MAINNET.value, addressOne, 18) ==
                Token(ChainId.MAINNET.value, addressOne, 1),
            equals(true));
      });
      test('true on reference equality', () {
        final token = Token(ChainId.MAINNET.value, addressOne, 18);
        expect(token == token, equals(true));
      });
      test('true even if name/symbol/decimals differ', () {
        final tokenA = Token(ChainId.MAINNET.value, addressOne, 9,
            symbol: 'abc', name: 'def');
        final tokenB = Token(ChainId.MAINNET.value, addressOne, 18,
            symbol: 'ghi', name: 'jkl');
        expect(tokenA == tokenB, equals(true));
      });

      group('#currencyEquals', (){
        test('test currency equals', (){
          final currentA = Token(1, addressOne, 18);
          final currentB = Token(1, addressOne, 18);
          expect(currentA, currentB);
        });
      });
    });
  });
}
